#pragma once


namespace Hypodermic
{

    namespace Tags
    {
        
        struct SelfRegistered {};
        struct NotSelfRegistered {};

        struct FallbackRegistration {};
        struct DefaultRegistration {};

    }

} // namespace Hypodermic